<?php
/***************************************************************************
                                smartfeed.php
                             -------------------
    begin                : Thurs Nov 24 2005
    copyright            : (C) 2000 The phpBB Group
    email                : mhamill@computer.org

    $Id: $

***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/

// Written by Mark D. Hamill, mhamill@computer.org
// This software is designed to work with phpBB Version 2.0.19

// This is the program that creates the RSS or Atom compliant newsfeed that will also include protected
// forums. An enhanced and slightly modified version of the the FeedCreator class, version 1.7.2 was used 
// to generate the various feeds. The modified version can be found at 
// http://blog.mypapit.net/2005/11/using-feedcreator-to-generate-atom-10-feeds.html.
// This version is needed because it supports Atom 1.0 and allow feeds to be created on the fly, 
// instead of stored to files.
//
// A companion program should be run first: smartfeed_url.php. This creates a URL that is
// used to invoke the feed. Use the URL created by this program in your newsreader. Do NOT try to run 
// this program without arguments!

define('IN_PHPBB', true);
$phpbb_root_path = './';

include($phpbb_root_path . 'extension.inc');
include($phpbb_root_path . 'common.'.$phpEx);
include($phpbb_root_path . 'includes/bbcode.'.$phpEx);
include($phpbb_root_path . 'includes/smartfeed_constants.'.$phpEx);
include($phpbb_root_path . 'includes/feedcreator.class.' . $phpEx); 

$userdata = session_pagestart($user_ip, PAGE_INDEX);
init_userprefs($userdata);

include($phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_smartfeed.' . $phpEx);

// Set up some needed variables
$error_msg = '';   // Used to capture any error message so the user can see they goofed. Errors are presented as a newsfeed item.
$error = false;    // Have any errors occurred?
$reset_user_lastvisit = false;    // If true, user_lastvisit is reset to current date and time; messages through that time will appear as read on the phpBB site
$phpBB_forum_user = false;  // Has this person registered with this board?
$smilies_path = preg_replace("/\//", "\/", $board_config['smilies_path']);

// Format Timezone for display
$timezone = explode('.', $board_config['board_timezone']);
$timezone = (count($timezone) > 1 && $timezone[count($timezone)-1] != 0) ? $lang[sprintf('%.1f', $board_config['board_timezone'])] : $lang[number_format($board_config['board_timezone'])];

$encrypted_pswd = ( !empty($_GET['p']) ) ? htmlspecialchars($_GET['p']) : '';
// + signs in $_GET['p'] seem to get translated to a space character, so put them back in
$encrypted_pswd = str_replace(' ','+',$encrypted_pswd);
$user_id = ( !empty($_GET['u']) ) ? intval($_GET['u']) : '';

// Arguments are required. If none, generate an error
if ($_SERVER['QUERY_STRING'] == '')
{
	$error_msg = $lang['smartfeed_no_arguments'];
	$error = true;
}

// Check for the p argument. p corresponds to the encrypted password created by smartfeed_url.php. The IP may also be appended and encrypted 
// if IP authentication was requested.
if ((!$error) && ($encrypted_pswd <> ''))
{
	// Check for the u argument. u corresponds to the user_id wanted and must be present if the p parameter is used 
	if ($user_id == '')
	{
		$error_msg = $lang['smartfeed_no_u_param'];
		$error = true;
	}
	else
	{
		$phpBB_forum_user = true;
	}
}
else
{
	// The u argument may not be used by itself
	if ($user_id <> '')
	{
		$error_msg .= $lang['smartfeed_no_p_param'];
		$error = true;
	}
}
	
// Validate user id, password and IP (if IP authentication requested)
if ((!$error) && $phpBB_forum_user) 
{
	// Make sure user_id exists in database
	$sql = "SELECT count(*) as count
		FROM " . USERS_TABLE . "
		WHERE user_id = '" . $user_id . "'";
		
	if ( !($result = $db->sql_query($sql)) )
	{
		$error_msg = $lang['smartfeed_user_table_count_error'] ;
		$error = true;
	}
	else
	{
		$row = $db->sql_fetchrow($result);
		
		if ($row['count'] == 0)
		{
			$error_msg = sprintf($lang['smartfeed_user_id_does_not_exist'], $user_id);
			$error = true;
		}
		else
		{
			// Validate encrypted password
			$sql = "SELECT user_password
				FROM " . USERS_TABLE . "
				WHERE user_id = '" . $user_id . "'";
			
			if ( !($result = $db->sql_query($sql)) )
			{
				$error_msg = $lang['smartfeed_user_table_password_error'];
				$error = true;
			}
			else
			{
				$row = $db->sql_fetchrow($result);
				
				// Decrypt password. It was encrypted in smartfeed_url with $dbpasswd from config.php. Note password in database is already
				// encoded but it is easy to break.
				include($phpbb_root_path . 'config.'.$phpEx);
				$encoded_pswd = decrypt($encrypted_pswd, $dbpasswd);
				unset($dbpasswd);
				
				// If IP Authentication was enabled, the encoded password is to the left of the ~ and the IP to the right of the ~
				$tilde = strpos ($encoded_pswd, '~');
				if ($tilde > 0)
				{
					$auth_ip = substr($encoded_pswd,$tilde+1);
					$encoded_pswd = substr($encoded_pswd,0,$tilde);
					$client_ip_parts = explode('.', $_SERVER['REMOTE_ADDR']);
					$source_ip_parts = explode('.', $auth_ip);
					// Show error message if requested from incorrect range of IP addresses
					if (!(($client_ip_parts[0] == $source_ip_parts[0]) && ($client_ip_parts[1] == $source_ip_parts[1]) && ($client_ip_parts[2] == $source_ip_parts[2])))
					{
						$error_msg = $lang['smartfeed_ip_auth_error'];
						$error = true;
					}
				}

				if ($encoded_pswd <> $row['user_password'])
				{
					$error_msg = sprintf($lang['smartfeed_bad_password_error'],  $encrypted_pswd, $user_id);
					$error = true;
				}
				else
				{
					// Check to see if last visit time should be reset
					$lastvisit = ( !empty($_GET['lastvisit']) ) ? htmlspecialchars($_GET['lastvisit']) : '';

					if (($lastvisit <> '') && ($lastvisit <> '1'))
					{
						$error_msg = $lang['smartfeed_lastvisit_param'];
						$error = true;
					}
					else
					{
						$reset_user_lastvisit = true;
					}	
				}			
			}
		}
	}
}

if (!$error)
{

	// Get all forum_ids that this user is allowed to access. This could be done more simply with SQL, but MySQL 3.x
	// does not support unions and intersections, so we need to do it the old fashioned way.
	
	$auth_restrict = ($phpBB_forum_user) ? AUTH_ALL  . ',' . AUTH_REG : AUTH_ALL;
	
	$sql = 'SELECT f.forum_id, f.forum_name, c.cat_order, f.forum_order
		FROM ' . FORUMS_TABLE . ' f, ' . CATEGORIES_TABLE . ' c
		WHERE f.cat_id = c.cat_id AND auth_read IN (' . $auth_restrict . ') 
		ORDER BY c.cat_order, f.forum_order';

	if ( !($result = $db->sql_query($sql)))
	{
		$error_msg = $lang['smartfeed_forum_access_reg'];
		$error = true;
	}

	if (!$error)
	{
		$allowed_forums = array();
		while ($row = $db->sql_fetchrow ($result)) 
		{ 
			$allowed_forums[] = $row['forum_id'];
		}
		$db->sql_freeresult ($result);
		
		if ($phpBB_forum_user)
		
		{
	
			$sql = 'SELECT DISTINCT a.forum_id, f.forum_name, c.cat_order, f.forum_order
				FROM ' . AUTH_ACCESS_TABLE . ' a, ' . USER_GROUP_TABLE . ' ug, ' . FORUMS_TABLE . ' f, ' . CATEGORIES_TABLE . ' c
				WHERE ug.user_id = ' . $user_id
				. ' AND ug.user_pending = 0 
				AND a.group_id = ug.group_id AND 
				a.forum_id = f.forum_id AND f.cat_id = c.cat_id';
		
			if ( !($result = $db->sql_query($sql)))
			{
				$error_msg = $lang['smartfeed_forum_access_priv'];
				$error = true;
			}
			
			if (!$error)
			{
				while ($row = $db->sql_fetchrow ($result)) 
				{ 
					$allowed_forums[] = $row['forum_id'];
				}
				$db->sql_freeresult ($result);
			}
			
		}
		
		if (!$error)
		{
			// Sort forums by forum_id and ensure there are no duplicates
			asort($allowed_forums);
			$allowed_forums = array_unique($allowed_forums);
		}
	}
}

if (!$error)
{

	// Now parse the URL field and get the forums the user wants to view
	
	$requested_forums = array();
	$query_string = ( !empty($_SERVER['QUERY_STRING']) ) ? $_SERVER['QUERY_STRING'] : '';
	$params = explode('&', $query_string);
	foreach ($params as $item)
	{
		if (substr($item,0,5) == 'forum')
			$requested_forums[] = substr($item,6);
	}
	asort($requested_forums);
	$requested_forums = array_unique($requested_forums);

	// The forums that will be fetched is the intersection of the requested and allowed forums. This prevents hacking
	// the URL to get feeds a user is not supposed to get. If no forums are specified on the URL field
	// then all forums that this user is authorized to access is assumed.
	
	if (count($requested_forums) > 0)
	{
		$fetched_forums = array_intersect($allowed_forums, $requested_forums);
	}
	else
	{
		$fetched_forums = $allowed_forums;
	}
	
	// Place forum numbers into a string suitable for use in a SQL "IN" statement

	$fetched_forums_str = implode(',',$fetched_forums);
	if ($fetched_forums_str <> '')
	{
		$fetched_forums_str = ' AND f.forum_id in (' . $fetched_forums_str . ')';
	}

	// Get the limit parameter. It limits the size of the newsfeed to a point in time from the present, either a day/hour/minute interval, 
	// or the time since the user's last visit. It should always exist.
	$limit = ( !empty($_GET['limit']) ) ? htmlspecialchars($_GET['limit']) : '';

	if ($limit == $lang['smartfeed_since_last_fetch_or_visit_value'])  // i.e. use the user's last visit and show all selected items after this time in the feed
	{
		if ($phpBB_forum_user)
		{
			// Logic to retrieve last fetched date
			$sql = 'SELECT user_lastvisit
				FROM ' . USERS_TABLE . "
				WHERE user_id = '" . $user_id . "'";
		
			if ( !($result = $db->sql_query($sql)))
			{
				$error_msg = $lang['smartfeed_user_error'];
				$error = true;
			}
			$row = $db->sql_fetchrow ($result);
			$user_lastvisit = $row['user_lastvisit'];
		}
		else
		{
			$user_lastvisit = 0;
		}
		
		// Check for a cookie. The cookie if it exists should contain the last newsfeed fetch time. Note that many if not
		// most newsreaders will ignore cookies. Some integrated with a browser, such as Sage for Firefox, should pick them
		// up.
		$cookie_time = ($_COOKIE['smartfeed'] <> '') ? strtotime(($_COOKIE['smartfeed'])) : 0;
		// Whichever time is greater, the user_lastvisit time in the phpBB users table, or the cookie value, sets a time value
		// We want all posts after this time.
		$last_fetch_time = max($user_lastvisit,  $cookie_time);
		if ($last_fetch_time == 0) // No cookie value and non-authenticated user could return millions of posts.
		{
			$last_fetch_time = DEFAULT_FETCH_TIME_LIMIT;
		}
		
		if (!$error)
		{
			$limit_text_str = ' AND p.post_time > ' . $last_fetch_time;
			$db->sql_freeresult ($result);
		}		
	}
	else if (($limit == $lang['smartfeed_last_week_value']) || ($limit == $lang['smartfeed_last_day_value']) || ($limit == $lang['smartfeed_last_12_hours_value']) || ($limit == $lang['smartfeed_last_6_hours_value']) || ($limit == $lang['smartfeed_last_3_hours_value']) || ($limit == $lang['smartfeed_last_1_hours_value']) || ($limit == $lang['smartfeed_last_30_minutes_value']) || ($limit == $lang['smartfeed_last_15_minutes_value']))
	{
		switch ($limit)
		{
			case $lang['smartfeed_last_week_value']:
				$limit_text = time() - (7 * 24 * 60 * 60);
				break;
			case $lang['smartfeed_last_day_value']:
				$limit_text = time() - (24 * 60 * 60);
				break;
			case $lang['smartfeed_last_12_hours_value']:
				$limit_text = time() - (12 * 60 * 60);
				break;
			case $lang['smartfeed_last_6_hours_value']:
				$limit_text = time() - (6 * 60 * 60);
				break;
			case $lang['smartfeed_last_3_hours_value']:
				$limit_text = time() - (3 * 60 * 60);
				break;
			case $lang['smartfeed_last_1_hours_value']:
				$limit_text = time() - (60 * 60);
				break;
			case $lang['smartfeed_last_30_minutes_value']:
				$limit_text = time() - (30 * 60);
				break;
			case $lang['smartfeed_last_15_minutes_value']:
				$limit_text = time() - (15 * 60);
		}
		$limit_text_str = ' AND p.post_time > ' . $limit_text;
	}
	else
	{
		// Bad parameter, trigger error
		$error_msg = $lang['smartfeed_limit_format_error'];
		$error = true;
	}
	
	// Validate the feed type next
	if (!$error)
	{
		$feed_type = ( !empty($_GET['feed_type']) ) ? htmlspecialchars($_GET['feed_type']) : '';

		if (!(($feed_type == SMARTFEED_ATOM_10_VALUE) || ($feed_type == SMARTFEED_RSS_20_VALUE) || ($feed_type == SMARTFEED_RSS_10_VALUE) || ($feed_type == SMARTFEED_RSS_091_VALUE)))
		{
			// Bad feed_type, trigger error
			$error_msg = $lang['smartfeed_feed_type_error'];
			$error = true;
		}
		else
		{
			if ($feed_type == SMARTFEED_RSS_091_VALUE && MAX_ITEMS > 0) 
			{
				$limit_str = ' LIMIT ' . min(MAX_ITEMS, 15);
			}
			else if (MAX_ITEMS > 0)
			{
				$limit_str = ' LIMIT ' . MAX_ITEMS;
			}
			else
			{
				$limit_str = '';
			}
		}
	}
	
	// Validate the sort by parameter
	if (!$error)
	{
		$order_by = ( !empty($_GET['sort_by']) ) ? htmlspecialchars($_GET['sort_by']) : '';
		if ($order_by == 'standard')
		{
			$order_by_str = 'c.cat_order, f.forum_order, lp.post_time desc, p.post_time';
		}
		else if ($order_by == 'postdate')
		{
			$order_by_str = 'p.post_time';
		}
		else
		{
			// Bad sort_bye, trigger error
			$error_msg = $lang['smartfeed_sort_by_error'];
			$error = true;
		}
	}
	
	// Validate the new topics only parameter. If it does not show, it is not true. Any value other than 1 is false.
	if (!$error)
	{
		$topics_only = ( !empty($_GET['topicsonly']) ) ? intval($_GET['topicsonly']) : '';
		if ($topics_only == '')
		{
			$topics_only_str = '';
		}
		else if ($topics_only == '1')
		{
			$topics_only_str = ' AND t.topic_time > ' . $limit_text;
		}
		else
		{
			// Bad sort_bye, trigger error
			$error_msg = $lang['smartfeed_topics_only_error'];
			$error = true;
		}
	}
			
	// Filter out users own posts, if they so elected
	if (!$error)
	{
		if ($phpBB_forum_user)
		{
			$removemine = ( !empty($_GET['removemine']) ) ? intval($_GET['removemine']) : '';
			if ($removemine == '')
			{
				$removemine_str = '';
			}
			else if ($removemine == '1')
			{
				$removemine_str = ' AND p.poster_id <> ' . $user_id ;
			}
			else
			{
				// Bad sort_bye, trigger error
				$error_msg = $lang['smartfeed_remove_yours_error'];
				$error = true;
			}
		}
	}
	
	if (!$error)
	{

		// If no new posts since the last fetch, return a 304 HTTP code. No point in regenerating a potentially very long newsfeed!
		
		$sql = "SELECT max(p.post_time) AS 'Last Post Timestamp'
			FROM " . POSTS_TABLE . ' lp, ' . FORUMS_TABLE . ' f, ' . TOPICS_TABLE . ' t, ' . CATEGORIES_TABLE . ' c, ' . POSTS_TABLE . ' p, ' . POSTS_TEXT_TABLE . ' pt, ' . USERS_TABLE . ' u
			WHERE lp.post_id = t.topic_last_post_id AND f.forum_id = t.forum_id AND
			f.cat_id = c.cat_id AND t.topic_id = p.topic_id AND p.post_id = pt.post_id AND p.poster_id = u.user_id ' . $removemine_str . $limit_text_str . $fetched_forums_str . $topics_only_str . '
			ORDER BY ' . $order_by_str . $limit_str; 
			
		if ( !($result = $db->sql_query($sql)))
		{
			$error_msg = $lang['smartfeed_retrieve_error'];
			$error = true;
		}
		else
		{
			$last_timestamp = $row['Last Post Timestamp'];
			doConditionalGet($last_timestamp); // Program may exit if no new content, returning 304 HTTP code 
		}
	}
			
	if (!$error)
	{
		// Create a list of messages for this user that presumably have not been seen after the date and time requested by the limit parameter.
		// Show only authorized forums.
		
		$sql = "SELECT c.cat_title, f.forum_name, t.topic_title, t.topic_id, p.post_time, u.username, pt.post_text, pt.post_subject, pt.post_id, 
			t.topic_first_post_id, u.user_viewemail, u.user_email, u.user_sig, pt.bbcode_uid, u.user_sig_bbcode_uid
			FROM " . POSTS_TABLE . ' lp, ' . FORUMS_TABLE . ' f, ' . TOPICS_TABLE . ' t, ' . CATEGORIES_TABLE . ' c, ' . POSTS_TABLE . ' p, ' . POSTS_TEXT_TABLE . ' pt, ' . USERS_TABLE . ' u
			WHERE lp.post_id = t.topic_last_post_id AND f.forum_id = t.forum_id AND
			f.cat_id = c.cat_id AND t.topic_id = p.topic_id AND p.post_id = pt.post_id AND p.poster_id = u.user_id ' . $removemine_str . $limit_text_str . $fetched_forums_str . $topics_only_str . '
			ORDER BY ' . $order_by_str . $limit_str; 
	
		if ( !($result = $db->sql_query($sql)))
		{
			$error_msg = $lang['smartfeed_retrieve_error'];
			$error = true;
		}
		else
		{
			$rss = new UniversalFeedCreator();
			$rss->useCached();
			$rss->title = $lang['smartfeed_feed_title'];
			$rss->description = $lang['smartfeed_feed_description'];
			$rss->link = ($feed_type == SMARTFEED_ATOM_10_VALUE) ? SITE_URL . 'smartfeed.' . $phpEx . '?' . $_SERVER['QUERY_STRING'] : SITE_URL . 'smartfeed.' . $phpEx . '?' . htmlspecialchars($_SERVER['QUERY_STRING']);
			$rss->syndicationURL = SITE_URL . 'smartfeed.' . $phpEx;
			$image = new FeedImage();
			$image->title = strip_tags($lang['smartfeed_image_title']); // Enhance to use default logo if not specified
			$image->url = SITE_URL . SMARTFEED_FEED_IMAGE_PATH;
			$image->link = ($feed_type == SMARTFEED_ATOM_10_VALUE) ? SITE_URL . 'smartfeed.' . $phpEx . '?' . $_SERVER['QUERY_STRING'] : SITE_URL . 'smartfeed.' . $phpEx . '?' . htmlspecialchars($_SERVER['QUERY_STRING']);
			$image->description = strip_tags($board_config['site_desc']);
			$rss->image = $image;
			
			// Some RSS 2.0 tags
			$rss->language = SMARTFEED_RFC1766_LANG;
			$rss->pubDate = time();
			$rss->ttl = TTL;
			$rss->copyright = ($lang['smartfeed_copyright'] <> '') ? $lang['smartfeed_copyright'] : '';
			$rss->editor = ($lang['smartfeed_editor'] <> '') ? $lang['smartfeed_editor'] : '';
			$rss->smartfeed_webmaster = ($lang['smartfeed_webmaster'] <> '') ? $lang['smartfeed_webmaster'] : '';
	
			while ($row = $db->sql_fetchrow ($result)) 
			{ 

				$item = new FeedItem();
				$item->title = ($row['post_subject'] == '') ? html_entity_decode($row['forum_name'] . ' :: ' . $row['topic_title']) : html_entity_decode($row['forum_name'] . ' :: ' . $row['post_subject']);
				if ($row['topic_first_post_id'] <> $row['post_id'])
				{
					$item->title .= ' ' . $lang['smartfeed_reply'];
				}
				$item->link = SITE_URL . 'viewtopic.' . $phpEx . '?' . POST_POST_URL . '=' . $row['post_id'] . '#' . $row['post_id'];
				$item->date = $row['post_time'];
				$item->pubDate = $row['post_time'];
				$item->source = SITE_URL;
				$item->category = $row['cat_title'];
				$item->guid = $item->link;
				$item->comments = SITE_URL . 'posting.' . $phpEx . '?mode=reply&t=' . $row['topic_id'];
				if (($feed_type==SMARTFEED_RSS_091_VALUE) || ($feed_type==SMARTFEED_RSS_20_VALUE)) // RSS 0.91 and 2.0 requires an email field to validate. Use a fake email address set in smartfeed_constants.php unless in user profile it says it is okay to show email address.
				{
					$item->author = ($row['user_viewemail'] == '1') ? $row['user_email'] . ' (' . $row['username']  . ')' : SMARTFEED_FAKE_EMAIL . ' (' . $row['username'] . ')';
				}					
				else
				{
					$item->author = $row['username'];
				}
				
				// Code from rss.php by Sascha Carlin, slightly modified 
				$post_text = '<em>' . $row['username'] . '</em> ' . $lang['smartfeed_wrote'] . ' ' . $lang['smartfeed_at'] . ' <em>' . date('d M Y h:i A ',$row['post_time']) . ' ' . $timezone . '</em>: ' . $row['post_text'];
				$post_text = str_replace("\n", "\n<br />\n", $post_text);
				$post_text = bbencode_second_pass($post_text, $row['bbcode_uid']);
				$post_text = smilies_pass($post_text);
				$post_text = preg_replace("/$smilies_path/", SMILIES_URL, $post_text);
				$post_text = make_clickable($post_text);
				// Variable reassignment and reformatting for user sig
				$user_sig = $row['user_sig'];
				$user_sig = bbencode_second_pass($user_sig, $post['user_sig_bbcode_uid']);
				$user_sig = smilies_pass($user_sig);
				$user_sig = preg_replace("/$smilies_path/", SMILIES_URL, $user_sig);
				$user_sig = make_clickable($user_sig);
				// End Code from rss.php by Sascha Carlin, slightly modified 

				if ( $user_sig != '' )
				{
					$user_sig = '<br />_________________<br />' . str_replace("\n", "\n<br />\n", $user_sig);
				}
				
				$item->link = SITE_URL . 'viewtopic.' . $phpEx . '?' . POST_POST_URL . '=' . $row['post_id'] . '#' . $row['post_id'];
				$item->description = $post_text. $user_sig;
				
				$rss->addItem($item); // Add this post to the feed

			}
			
			// If Last Fetch was selected, set a cookie for the current time, so that posts before the cookie date will not be retrieved the next time this script is called
			// Note that cookies are often not handled by feed reader appliations, particularly those installed on the desktop like FeedReader. However, browser based
			// extensions like Sage for Firefox will set cookies.
			if ($limit == $lang['smartfeed_since_last_fetch_or_visit_value'])
			{
				$cookie_set = setcookie('smartfeed', date('r', time()), time()+60*60*24*7, $board_config['cookie_path'], $board_config['cookie_domain'], $board_config['cookie_secure']);
			}
			$rss->outputFeed($feed_type);				
			$db->sql_freeresult ($result);
		}
		
		// If there were no problems, reset the user's last visit date to the current time and date if they so elected. By doing so
		// when they next hit the phpBB board it should appear as if all messages were read.
		
		if ($reset_user_lastvisit)
		{
			$sql = "UPDATE " . USERS_TABLE . "
				SET USER_LASTVISIT = " . time() . 
				" WHERE user_id = '" . $user_id . "'";
				
			if ( !($result = $db->sql_query($sql)) )
			{
				$error_msg = $lang['smartfeed_reset_error'];
				$error = true;
			}
		}
	}
}

if ($error)
{

	// Send the error in the newsfeed itself, but with no other items
	
	$rss = new UniversalFeedCreator();
	$rss->useCached();
	$rss->title = $lang['smartfeed_feed_title'];
	$rss->description = $lang['smartfeed_feed_description'];
	$rss->link = SITE_URL . 'smartfeed.' . $phpEx;
	$rss->syndicationURL = SITE_URL . 'smartfeed.' . $phpEx;
	
	// Some RSS 2.0 tags
	$rss->language = SMARTFEED_RFC1766_LANG;
	$rss->pubDate = time();
	$rss->ttl = TTL;
	$rss->copyright = ($lang['smartfeed_copyright'] <> '') ? $lang['smartfeed_copyright'] : '';
	$rss->editor = ($lang['smartfeed_editor'] <> '') ? $lang['smartfeed_editor'] : '';
	$rss->smartfeed_webmaster = ($lang['smartfeed_webmaster'] <> '') ? $lang['smartfeed_webmaster'] : '';
	
	$item = new FeedItem();
	$item->title = $lang['smartfeed_error_title'];
	$item->link = SITE_URL . 'smartfeed.' . $phpEx;
	$item->source = SITE_URL;
	$item->description = $lang['smartfeed_error_introduction'] . $error_msg;
	
	$rss->addItem($item);
	$rss->outputFeed($feed_type);				
}

function decrypt($string, $key) 
{

	$result = '';
	$string = base64_decode($string);
	
	for($i=0; $i<strlen($string); $i++) 
	{
		$char = substr($string, $i, 1);
		$keychar = substr($key, ($i % strlen($key))-1, 1);
		$char = chr(ord($char)-ord($keychar));
		$result.=$char;
	}
	return $result;
	
}
 
function doConditionalGet($timestamp) 
{

	// This code found at: http://simon.incutio.com/archive/2003/04/23/conditionalGet
	$last_modified = substr(date('r', $timestamp), 0, -5).'GMT';
	$etag = '"'.md5($last_modified).'"';
	// Send the headers
	header("Last-Modified: $last_modified");
	header("ETag: $etag");
	// See if the client has provided the required headers
	$if_modified_since = isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) ?
		stripslashes($_SERVER['HTTP_IF_MODIFIED_SINCE']) :
		false;
	$if_none_match = isset($_SERVER['HTTP_IF_NONE_MATCH']) ?
		stripslashes($_SERVER['HTTP_IF_NONE_MATCH']) : 
		false;
	if (!$if_modified_since && !$if_none_match) 
	{
		return;
	}
	// At least one of the headers is there - check them
	if ($if_none_match && $if_none_match != $etag) 
	{
		return; // etag is there but doesn't match
	}
	if ($if_modified_since && $if_modified_since != $last_modified) 
	{
		return; // if-modified-since is there but doesn't match
	}
	// Nothing has changed since their last request - serve a 304 and exit
	header('HTTP/1.0 304 Not Modified');
	exit;
	
}
?>
